# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from apitax.api.models.base_model_ import Model
from apitax.api import util


class Script(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, content=None, name=None):  # noqa: E501
        """Script - a model defined in Swagger

        :param content: The content of this Script.  # noqa: E501
        :type content: str
        :param name: The name of this Script.  # noqa: E501
        :type name: str
        """
        self.swagger_types = {
            'content': str,
            'name': str
        }

        self.attribute_map = {
            'content': 'content',
            'name': 'name'
        }

        self._content = content
        self._name = name

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Script of this Script.  # noqa: E501
        :rtype: Script
        """
        return util.deserialize_model(dikt, cls)

    @property
    def content(self):
        """Gets the content of this Script.


        :return: The content of this Script.
        :rtype: str
        """
        return self._content

    @content.setter
    def content(self, content):
        """Sets the content of this Script.


        :param content: The content of this Script.
        :type content: str
        """

        self._content = content

    @property
    def name(self):
        """Gets the name of this Script.


        :return: The name of this Script.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Script.


        :param name: The name of this Script.
        :type name: str
        """

        self._name = name



from apitaxcore.models.Credentials import Credentials
from apitaxcore.drivers.Driver import Driver
from apitaxcore.utilities.Json import read
from apitaxcore.models.State import State
from apitaxcore.utilities.Files import getPath


class BasicAuth(Driver):
    def __init__(self):
        super().__init__()
        self.users = read(getPath(State.paths['root'] + "/app/users.json"))

    def getDriverName(self) -> str:
        return "Basic Auth"

    def getDriverDescription(self) -> str:
        return "Provides out of the box authentication"

    def getDriverHelpEndpoint(self) -> str:
        return "coming soon"

    def getDriverTips(self) -> str:
        return "coming soon"

    def isDriverAuthenticatable(self) -> bool:
        return True

    def isApitaxAuthBasedOnApiAuth(self) -> bool:
        return False

    def authenticateApitax(self, credentials: Credentials = None) -> str:
        try:
            if credentials.password == self.users[credentials.username]['password']:
                return self.users[credentials.username]['role']
        except:
            return None
        return None




def isRole(identity, role='user'):
    return identity['role'] == role


def hasAccess(identity, role='user'):
    has = convertRoleToNumber(identity['role'])
    required = convertRoleToNumber(role)
    if has >= required:
        return True
    return False


def convertRoleToNumber(role):
    if(role == 'user'):
        return 1
    if(role == 'developer'):
        return 2
    if(role == 'admin'):
        return 3
    return 0



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from apitax.api.models.base_model_ import Model
from apitax.api.models.script import Script  # noqa: F401,E501
from apitax.api import util


class ScriptSave(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, script=None):  # noqa: E501
        """ScriptSave - a model defined in Swagger

        :param script: The script of this ScriptSave.  # noqa: E501
        :type script: Script
        """
        self.swagger_types = {
            'script': Script
        }

        self.attribute_map = {
            'script': 'script'
        }

        self._script = script

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The script_save of this ScriptSave.  # noqa: E501
        :rtype: ScriptSave
        """
        return util.deserialize_model(dikt, cls)

    @property
    def script(self):
        """Gets the script of this ScriptSave.


        :return: The script of this ScriptSave.
        :rtype: Script
        """
        return self._script

    @script.setter
    def script(self, script):
        """Sets the script of this ScriptSave.


        :param script: The script of this ScriptSave.
        :type script: Script
        """
        if script is None:
            raise ValueError("Invalid value for `script`, must not be `None`")  # noqa: E501

        self._script = script



# coding: utf-8

from __future__ import absolute_import

from flask import json
from six import BytesIO

from apitax.api.models.error_response import ErrorResponse  # noqa: E501
from apitax.api.models.response import Response  # noqa: E501
from apitax.api.models.script_create import ScriptCreate  # noqa: E501
from apitax.api.models.script_delete import ScriptDelete  # noqa: E501
from apitax.api.models.script_rename import ScriptRename  # noqa: E501
from apitax.api.models.script_save import ScriptSave  # noqa: E501
from apitax.api.test import BaseTestCase


class TestScriptaxController(BaseTestCase):
    """ScriptaxController integration test stubs"""

    def test_create_driver_script(self):
        """Test case for create_driver_script

        Create a new script
        """
        script_create = ScriptCreate()
        response = self.client.open(
            '/apitax/2/drivers/{driver}/scriptax/scripts'.format(driver='driver_example'),
            method='POST',
            data=json.dumps(script_create),
            content_type='application/json')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_delete_driver_script(self):
        """Test case for delete_driver_script

        Delete a script
        """
        script_delete = ScriptDelete()
        response = self.client.open(
            '/apitax/2/drivers/{driver}/scriptax/scripts'.format(driver='driver_example'),
            method='DELETE',
            data=json.dumps(script_delete),
            content_type='application/json')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_driver_script(self):
        """Test case for get_driver_script

        Retrieve the contents of a script
        """
        query_string = [('script', 'script_example')]
        response = self.client.open(
            '/apitax/2/drivers/{driver}/scriptax/scripts'.format(driver='driver_example'),
            method='GET',
            content_type='application/json',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_driver_script_catalog(self):
        """Test case for get_driver_script_catalog

        Retrieve the script catalog
        """
        response = self.client.open(
            '/apitax/2/drivers/{driver}/scriptax/catalog'.format(driver='driver_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_rename_driver_script(self):
        """Test case for rename_driver_script

        Rename a script
        """
        script_rename = ScriptRename()
        response = self.client.open(
            '/apitax/2/drivers/{driver}/scriptax/scripts'.format(driver='driver_example'),
            method='PATCH',
            data=json.dumps(script_rename),
            content_type='application/json')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_save_driver_script(self):
        """Test case for save_driver_script

        Save a script
        """
        script_save = ScriptSave()
        response = self.client.open(
            '/apitax/2/drivers/{driver}/scriptax/scripts'.format(driver='driver_example'),
            method='PUT',
            data=json.dumps(script_save),
            content_type='application/json')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))


if __name__ == '__main__':
    import unittest
    unittest.main()



# System imports
import sys
import os
import inspect
from pathlib import Path

# Default imports
import click

# Application imports
from apitaxcore.config.Config import Config as ConfigConsumer

from apitaxcore.models.Options import Options
from apitaxcore.flow.LoadedDrivers import LoadedDrivers
from apitaxcore.utilities.Files import getRoot

from apitaxcore.logs.Log import Log
from apitaxcore.logs.BufferedLog import BufferedLog
from apitaxcore.logs.StandardLog import StandardLog

from apitaxcore.models.State import State


class Setup:
    def __init__(self, passedArgs: list = []):

        if len(passedArgs) == 0:
            self.args = sys.argv[1:]
        else:
            self.args = passedArgs

        self.usage = 'cli'

        self.debug = False
        self.sensitive = False

        self.reloader = False
        self.watcher = False

        self.username = ''
        self.password = ''

        self.command = ''
        self.script = ''

        self.build = True

        self.doLog = True
        if State.paths['log'] != "":
            self.logPath = State.paths['log']
        else:
            self.logPath = '/logs/apitax.log'
        self.logColorize = True
        self.logPrefixes = True
        self.logHumanReadable = False

        # print(getRootPath('/config.txt'))
        if State.paths['config'] != "":
            configFile = State.paths['config']
        else:
            configFile = getRoot('/config.txt')
        self.config = ConfigConsumer.read()
        self.config.path = str(Path(os.path.dirname(os.path.abspath(inspect.stack()[0][1]))).resolve())
        # print(getRootPath())

        if self.config.has('default-mode'):
            self.usage = self.config.get('default-mode')

        if self.config.has('log'):
            self.doLog = self.config.get('log')

        if self.config.has('log-file'):
            self.logPath = self.config.get('log-file')

        if self.config.has('log-colorize'):
            self.logColorize = self.config.get('log-colorize')

        if self.config.has('log-human-readable'):
            self.logHumanReadable = self.config.get('log-human-readable')

        if self.config.has('log-prefixes'):
            self.logPrefixes = self.config.get('log-prefixes')

        if self.logPath[:1] != '/':
            self.logPath = '/' + self.logPath

        # self.logPath = getRootPath(self.logPath)

        if self.config.has('log-buffered') and self.config.get('log-buffered'):
            logDriver = BufferedLog()
        else:
            logDriver = StandardLog()

        self.log = Log(logDriver, logFile=self.logPath, doLog=self.doLog, logColorize=self.logColorize,
                       logPrefixes=self.logPrefixes,
                       logHumanReadable=self.logHumanReadable)

        self.log.log('')
        self.log.log('')
        self.log.log(">>>  Apitax - Combining the power of Commandtax and Scriptax")
        self.log.log('')
        self.log.log('')

        if '--cli' in self.args:
            self.usage = 'cli'
        elif '--api' in self.args:
            self.usage = 'api'
        elif '--grammar-test' in self.args:
            self.usage = 'grammar-test'
        elif '--feature-test' in self.args:
            self.usage = 'feature-test'

        if '--debug' in self.args:
            self.debug = True

        if '--sensitive' in self.args:
            self.sensitive = True

        if '--reloader' in self.args:
            self.reloader = True

        if '-u' in self.args:
            self.username = self.args[self.args.index('-u') + 1]

        if '-p' in self.args:
            self.password = click.prompt('Enter Your Password', hide_input=True)

        if '-s' in self.args:
            self.script = self.args[self.args.index('-s') + 1]

        if '-r' in self.args:
            self.command = self.args[self.args.index('-r') + 1]
        elif self.script == '' and self.usage == 1:
            self.command = click.prompt('R')

        # This is to turn the '-s' flag into a command behind the scenes
        if self.script != '':
            self.command = 'script ' + self.script

        self.options = Options(debug=self.debug, sensitive=self.sensitive)

        self.loggingSettings = self.log.getLoggerSettings()

        self.log.log('>> Runtime Settings:')

        self.log.log('    * Paths')

        self.log.log('      * Root:   ' + State.paths['root'])

        self.log.log('      * Config: ' + configFile)

        self.log.log('      * Log:    ' + str(self.loggingSettings.get('path')))

        self.log.log('      * Apitax: ' + State.paths['apitax'])

        self.log.log('    * Debug: ' + str(self.debug))

        self.log.log('    * Sensitive: ' + str(self.sensitive))

        self.log.log('    * Operating out of: ' + str(self.config.path))

        self.log.log('    * Logging: ' + str(self.loggingSettings.get('doLog')))

        if self.loggingSettings.get('doLog'):
            self.log.log('      * Colorize CLI: ' + str(self.loggingSettings.get('colorize')))

        self.log.log('')
        self.log.log('')

        if self.options.debug:
            self.log.log('>> Setting up AppState')
            self.log.log('')
            self.log.log('')

        # Setup App State
        State.config = self.config
        State.options = self.options
        State.log = self.log
        if State.paths['log'] == "":
            State.paths['log'] = self.logPath
        if State.paths['root'] == "":
            State.paths['root'] = getRoot()
        if State.paths['apitax'] == "":
            State.paths['apitax'] = str(Path(os.path.dirname(os.path.abspath(inspect.stack()[0][1]))).resolve())
        if State.paths['config'] == "":
            State.paths['config'] = configFile

        if (self.options.debug):
            self.log.log('>> Loading Drivers')
            self.log.log('')

        self.log.getLoggerDriver().outputLog()

    def load(self):
        drivers = []
        if self.config.has('drivers-use'):
            drivers += self.config.getAsList('drivers-use')

        for driver in drivers:
            LoadedDrivers.load(driver)

        if self.options.debug:
            self.log.log('>> Finished Loading Drivers')
            self.log.log('')
            self.log.log('')

        self.log.getLoggerDriver().outputLog()



from apitax.flow.Project import Project as BaseProject


class Project(BaseProject):
    def setup(self):
        from apitaxcore.models.State import State
        State.paths['root'] = 'D:/Programming/Projects/Apitax/Apitax'  # just /app for docker
        State.paths['config'] = State.paths['root'] + '/app/config.txt'
        State.paths['log'] = None

    def loadDrivers(self):
        from apitaxcore.drivers.Drivers import Drivers
        from apitaxdrivers.Github import GithubDriver
        Drivers.add("github", GithubDriver())



import connexion
import six

from apitax.api.models.error_response import ErrorResponse  # noqa: E501
from apitax.api.models.response import Response  # noqa: E501
from apitax.api import util

from apitaxcore.flow.LoadedDrivers import LoadedDrivers
from apitaxcore.drivers.Driver import Driver
from apitaxcore.flow.responses.ApitaxResponse import ApitaxResponse
from apitax.api.utilities.Lifecycle import redirectIfUnauthorized, errorIfUnauthorized
from flask_jwt_extended import (create_access_token, create_refresh_token, jwt_required, jwt_refresh_token_required,
                                get_jwt_identity, get_raw_jwt, get_jwt_claims)

@jwt_required
def get_driver_blacklist(driver):  # noqa: E501
    """Retrieve the blacklist in the driver

    Retrieve the blacklist in the driver # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """

    response = errorIfUnauthorized(role='admin')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    response.body.add({'blacklist': driver.getDriverBlacklist()})

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def get_driver_config(driver):  # noqa: E501
    """Retrieve the config of a loaded driver

    Retrieve the config of a loaded driver # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """

    response = errorIfUnauthorized(role='admin')
    if response:
        return response
    else:
        response = ApitaxResponse()

    # TODO: This needs an implementation, but likely requires a change to configs in apitaxcore

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def get_driver_list():  # noqa: E501
    """Retrieve the catalog of drivers

    Retrieve the catalog of drivers # noqa: E501


    :rtype: Response
    """

    response = errorIfUnauthorized(role='admin')
    if response:
        return response
    else:
        response = ApitaxResponse()

    response.body.add({'drivers': LoadedDrivers.drivers})

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def get_driver_status(driver):  # noqa: E501
    """Retrieve the status of a loaded driver

    Retrieve the status of a loaded driver # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """

    response = errorIfUnauthorized(role='admin')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    response.body.add({'name': driver.getDriverName()})
    response.body.add({'description': driver.getDriverDescription()})
    response.body.add({'tips': driver.getDriverTips()})
    response.body.add({'help': driver.getDriverHelpEndpoint()})
    response.body.add({'minimum-role': driver.getDriverMinimumRole()})

    options = {}
    options['configurable'] = driver.isDriverConfigurable()
    options['authenticatable'] = driver.isDriverAuthenticatable()
    options['role-restricted'] = driver.isDriverRoleRestricted()
    options['whitelisted'] = driver.isDriverWhitelisted()
    options['blacklisted'] = driver.isDriverBlacklisted()
    response.body.add({'options': options})

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def get_driver_whitelist(driver):  # noqa: E501
    """Retrieve the whitelist in the driver

    Retrieve the whitelist in the driver # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """

    response = errorIfUnauthorized(role='admin')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    response.body.add({'whitelist': driver.getDriverWhitelist()})

    return Response(status=200, body=response.getResponseBody())



# coding: utf-8

from __future__ import absolute_import

from flask import json
from six import BytesIO

from apitax.api.models.error_response import ErrorResponse  # noqa: E501
from apitax.api.models.response import Response  # noqa: E501
from apitax.api.test import BaseTestCase


class TestApiController(BaseTestCase):
    """ApiController integration test stubs"""

    def test_get_driver_api_catalog(self):
        """Test case for get_driver_api_catalog

        Retrieve the api catalog
        """
        response = self.client.open(
            '/apitax/2/drivers/{driver}/api/catalog'.format(driver='driver_example'),
            method='GET',
            content_type='application/json')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_driver_api_status(self):
        """Test case for get_driver_api_status

        Retrieve the status of an api backing a driver
        """
        response = self.client.open(
            '/apitax/2/drivers/{driver}/api/status'.format(driver='driver_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))


if __name__ == '__main__':
    import unittest
    unittest.main()



import connexion
import six

from apitax.api.models.error_response import ErrorResponse  # noqa: E501
from apitax.api.models.response import Response  # noqa: E501
from apitax.api import util

from apitaxcore.flow.LoadedDrivers import LoadedDrivers
from apitaxcore.drivers.Driver import Driver
from apitaxcore.flow.responses.ApitaxResponse import ApitaxResponse
from apitax.api.utilities.Lifecycle import redirectIfUnauthorized, errorIfUnauthorized
from flask_jwt_extended import (create_access_token, create_refresh_token, jwt_required, jwt_refresh_token_required,
                                get_jwt_identity, get_raw_jwt, get_jwt_claims)


@jwt_required
def get_driver_api_catalog(driver):  # noqa: E501
    """Retrieve the api catalog

    Retrieve the api catalog # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """
    response = errorIfUnauthorized(role='developer')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    response.body.add(driver.getApiEndpointCatalog())

    return Response(status=200, body=response.getResponseBody())


@jwt_required
def get_driver_api_status(driver):  # noqa: E501
    """Retrieve the status of an api backing a driver

    Retrieve the status of an api backing a driver # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """
    response = errorIfUnauthorized(role='developer')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    response.body.add({"format": driver.getApiFormat()})
    response.body.add({"description": driver.getApiDescription()})
    response.body.add({"status": driver.getApiStatus()})
    response.body.add({"auth-type": driver.getApiAuthType()})

    endpoints = {}
    endpoints['base'] = driver.getApiBaseEndpoint()
    endpoints['catalog'] = driver.getApiCatalogEndpoint()
    endpoints['auth'] = driver.getApiAuthEndpoint()
    response.body.add({'endpoints': endpoints})

    options = {}
    options['authenticatable'] = driver.isApiAuthenticated()
    options['authentication-separate'] = driver.isApiAuthenticationSeparateRequest()
    options['cataloggable'] = driver.isApiCataloggable()
    options['tokenable'] = driver.isApiTokenable()
    response.body.add({'options': options})

    return Response(status=200, body=response.getResponseBody())



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from apitax.api.models.base_model_ import Model
from apitax.api.models.command import Command  # noqa: F401,E501
from apitax.api.models.user_auth import UserAuth  # noqa: F401,E501
from apitax.api import util


class Execute(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, command=None, auth=None):  # noqa: E501
        """Execute - a model defined in Swagger

        :param command: The command of this Execute.  # noqa: E501
        :type command: Command
        :param auth: The auth of this Execute.  # noqa: E501
        :type auth: UserAuth
        """
        self.swagger_types = {
            'command': Command,
            'auth': UserAuth
        }

        self.attribute_map = {
            'command': 'command',
            'auth': 'auth'
        }

        self._command = command
        self._auth = auth

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The execute of this Execute.  # noqa: E501
        :rtype: Execute
        """
        return util.deserialize_model(dikt, cls)

    @property
    def command(self):
        """Gets the command of this Execute.


        :return: The command of this Execute.
        :rtype: Command
        """
        return self._command

    @command.setter
    def command(self, command):
        """Sets the command of this Execute.


        :param command: The command of this Execute.
        :type command: Command
        """
        if command is None:
            raise ValueError("Invalid value for `command`, must not be `None`")  # noqa: E501

        self._command = command

    @property
    def auth(self):
        """Gets the auth of this Execute.


        :return: The auth of this Execute.
        :rtype: UserAuth
        """
        return self._auth

    @auth.setter
    def auth(self, auth):
        """Sets the auth of this Execute.


        :param auth: The auth of this Execute.
        :type auth: UserAuth
        """

        self._auth = auth
